# This file was auto-generated by Fern from our API Definition.

from ...core.api_error import ApiError
from ..types.error_message import ErrorMessage


class BadRequestError(ApiError):
    def __init__(self, body: ErrorMessage):
        super().__init__(status_code=400, body=body)



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing_extensions
import typing
from .entity_id_without_agent import EntityIdWithoutAgent
from ...core.serialization import FieldMetadata
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class ConversationSummary(UniversalBaseModel):
    action_ids: typing_extensions.Annotated[typing.List[EntityIdWithoutAgent], FieldMetadata(alias="actionIds")] = (
        pydantic.Field()
    )
    """
    The IDs of the actions that were taken by Maven in the conversation
    """

    incomplete_action_ids: typing_extensions.Annotated[
        typing.List[EntityIdWithoutAgent], FieldMetadata(alias="incompleteActionIds")
    ] = pydantic.Field()
    """
    The IDs of the actions that were taken by Maven but not completed in the conversation. Occurs when the user is shown an action form but does not submit it.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

NumericConversationField = typing.Union[
    typing.Literal["ThumbsUpCount", "ThumbsDownCount", "UserMessageCount", "HandleTime", "FirstResponseTime"],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ...core.pydantic_utilities import UniversalBaseModel
import typing
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic
import typing_extensions
from ...conversation.types.numeric_conversation_field import NumericConversationField
from ...core.serialization import FieldMetadata
from ...conversation.types.conversation_field import ConversationField


class ConversationMetric_Count(UniversalBaseModel):
    """
    Defines the metric to be calculated for a column or chart.
    Only numeric fields are supported, except for ConversationCount and ConversationDistinctCount, which can be applied to any field.
    """

    type: typing.Literal["count"] = "count"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ConversationMetric_Sum(UniversalBaseModel):
    """
    Defines the metric to be calculated for a column or chart.
    Only numeric fields are supported, except for ConversationCount and ConversationDistinctCount, which can be applied to any field.
    """

    type: typing.Literal["sum"] = "sum"
    target_field: typing_extensions.Annotated[NumericConversationField, FieldMetadata(alias="targetField")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ConversationMetric_Average(UniversalBaseModel):
    """
    Defines the metric to be calculated for a column or chart.
    Only numeric fields are supported, except for ConversationCount and ConversationDistinctCount, which can be applied to any field.
    """

    type: typing.Literal["average"] = "average"
    target_field: typing_extensions.Annotated[NumericConversationField, FieldMetadata(alias="targetField")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ConversationMetric_Min(UniversalBaseModel):
    """
    Defines the metric to be calculated for a column or chart.
    Only numeric fields are supported, except for ConversationCount and ConversationDistinctCount, which can be applied to any field.
    """

    type: typing.Literal["min"] = "min"
    target_field: typing_extensions.Annotated[NumericConversationField, FieldMetadata(alias="targetField")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ConversationMetric_Max(UniversalBaseModel):
    """
    Defines the metric to be calculated for a column or chart.
    Only numeric fields are supported, except for ConversationCount and ConversationDistinctCount, which can be applied to any field.
    """

    type: typing.Literal["max"] = "max"
    target_field: typing_extensions.Annotated[NumericConversationField, FieldMetadata(alias="targetField")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ConversationMetric_Percentile(UniversalBaseModel):
    """
    Defines the metric to be calculated for a column or chart.
    Only numeric fields are supported, except for ConversationCount and ConversationDistinctCount, which can be applied to any field.
    """

    type: typing.Literal["percentile"] = "percentile"
    percentile: float
    target_field: typing_extensions.Annotated[NumericConversationField, FieldMetadata(alias="targetField")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ConversationMetric_Median(UniversalBaseModel):
    """
    Defines the metric to be calculated for a column or chart.
    Only numeric fields are supported, except for ConversationCount and ConversationDistinctCount, which can be applied to any field.
    """

    type: typing.Literal["median"] = "median"
    target_field: typing_extensions.Annotated[NumericConversationField, FieldMetadata(alias="targetField")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ConversationMetric_DistinctCount(UniversalBaseModel):
    """
    Defines the metric to be calculated for a column or chart.
    Only numeric fields are supported, except for ConversationCount and ConversationDistinctCount, which can be applied to any field.
    """

    type: typing.Literal["distinctCount"] = "distinctCount"
    target_field: typing_extensions.Annotated[ConversationField, FieldMetadata(alias="targetField")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


ConversationMetric = typing.Union[
    ConversationMetric_Count,
    ConversationMetric_Sum,
    ConversationMetric_Average,
    ConversationMetric_Min,
    ConversationMetric_Max,
    ConversationMetric_Percentile,
    ConversationMetric_Median,
    ConversationMetric_DistinctCount,
]



# This file was auto-generated by Fern from our API Definition.

from .bar_chart_request import BarChartRequest
from .bar_chart_response import BarChartResponse
from .cell_data import CellData, CellData_Double, CellData_Long, CellData_Millisecond, CellData_String
from .chart_response import ChartResponse, ChartResponse_BarChart, ChartResponse_DateHistogram, ChartResponse_PieChart
from .column_definition_base import ColumnDefinitionBase
from .conversation_analytics_request import ConversationAnalyticsRequest
from .conversation_average import ConversationAverage
from .conversation_basic_metric import ConversationBasicMetric
from .conversation_chart_request import (
    ConversationChartRequest,
    ConversationChartRequest_BarChart,
    ConversationChartRequest_DateHistogram,
    ConversationChartRequest_PieChart,
)
from .conversation_column_definition import ConversationColumnDefinition
from .conversation_count import ConversationCount
from .conversation_distinct_count import ConversationDistinctCount
from .conversation_group_by import ConversationGroupBy
from .conversation_max import ConversationMax
from .conversation_median import ConversationMedian
from .conversation_metric import (
    ConversationMetric,
    ConversationMetric_Average,
    ConversationMetric_Count,
    ConversationMetric_DistinctCount,
    ConversationMetric_Max,
    ConversationMetric_Median,
    ConversationMetric_Min,
    ConversationMetric_Percentile,
    ConversationMetric_Sum,
)
from .conversation_min import ConversationMin
from .conversation_numeric_metric import ConversationNumericMetric
from .conversation_percentile import ConversationPercentile
from .conversation_row import ConversationRow
from .conversation_sum import ConversationSum
from .conversation_table_request import ConversationTableRequest
from .conversation_table_response import ConversationTableResponse
from .date_histogram_request import DateHistogramRequest
from .date_histogram_response import DateHistogramResponse
from .feedback_analytics_request import FeedbackAnalyticsRequest
from .feedback_column_definition import FeedbackColumnDefinition
from .feedback_count import FeedbackCount
from .feedback_distinct_count import FeedbackDistinctCount
from .feedback_group_by import FeedbackGroupBy
from .feedback_metric import FeedbackMetric, FeedbackMetric_Count, FeedbackMetric_DistinctCount
from .feedback_row import FeedbackRow
from .feedback_table_request import FeedbackTableRequest
from .feedback_table_response import FeedbackTableResponse
from .field_value import (
    FieldValue,
    FieldValue_Boolean,
    FieldValue_DateTime,
    FieldValue_Double,
    FieldValue_EntityId,
    FieldValue_Long,
    FieldValue_Range,
    FieldValue_String,
)
from .group_by_base import GroupByBase
from .labeled_point import LabeledPoint
from .pie_chart_request import PieChartRequest
from .pie_chart_response import PieChartResponse
from .range import Range
from .row_base import RowBase
from .series import Series
from .table_response_base import TableResponseBase
from .time_data_point import TimeDataPoint
from .time_interval import TimeInterval
from .time_series import TimeSeries

__all__ = [
    "BarChartRequest",
    "BarChartResponse",
    "CellData",
    "CellData_Double",
    "CellData_Long",
    "CellData_Millisecond",
    "CellData_String",
    "ChartResponse",
    "ChartResponse_BarChart",
    "ChartResponse_DateHistogram",
    "ChartResponse_PieChart",
    "ColumnDefinitionBase",
    "ConversationAnalyticsRequest",
    "ConversationAverage",
    "ConversationBasicMetric",
    "ConversationChartRequest",
    "ConversationChartRequest_BarChart",
    "ConversationChartRequest_DateHistogram",
    "ConversationChartRequest_PieChart",
    "ConversationColumnDefinition",
    "ConversationCount",
    "ConversationDistinctCount",
    "ConversationGroupBy",
    "ConversationMax",
    "ConversationMedian",
    "ConversationMetric",
    "ConversationMetric_Average",
    "ConversationMetric_Count",
    "ConversationMetric_DistinctCount",
    "ConversationMetric_Max",
    "ConversationMetric_Median",
    "ConversationMetric_Min",
    "ConversationMetric_Percentile",
    "ConversationMetric_Sum",
    "ConversationMin",
    "ConversationNumericMetric",
    "ConversationPercentile",
    "ConversationRow",
    "ConversationSum",
    "ConversationTableRequest",
    "ConversationTableResponse",
    "DateHistogramRequest",
    "DateHistogramResponse",
    "FeedbackAnalyticsRequest",
    "FeedbackColumnDefinition",
    "FeedbackCount",
    "FeedbackDistinctCount",
    "FeedbackGroupBy",
    "FeedbackMetric",
    "FeedbackMetric_Count",
    "FeedbackMetric_DistinctCount",
    "FeedbackRow",
    "FeedbackTableRequest",
    "FeedbackTableResponse",
    "FieldValue",
    "FieldValue_Boolean",
    "FieldValue_DateTime",
    "FieldValue_Double",
    "FieldValue_EntityId",
    "FieldValue_Long",
    "FieldValue_Range",
    "FieldValue_String",
    "GroupByBase",
    "LabeledPoint",
    "PieChartRequest",
    "PieChartResponse",
    "Range",
    "RowBase",
    "Series",
    "TableResponseBase",
    "TimeDataPoint",
    "TimeInterval",
    "TimeSeries",
]



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing_extensions
from ...conversation.types.numeric_conversation_field import NumericConversationField
from ...core.serialization import FieldMetadata
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class ConversationNumericMetric(UniversalBaseModel):
    target_field: typing_extensions.Annotated[NumericConversationField, FieldMetadata(alias="targetField")] = (
        pydantic.Field()
    )
    """
    Numeric field to apply the metric to.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
import typing_extensions
import datetime as dt
from ...core.serialization import FieldMetadata
from ...commons.types.feedback_type import FeedbackType
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class FeedbackFilter(UniversalBaseModel):
    search: typing.Optional[str] = None
    created_after: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="createdAfter")] = None
    created_before: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="createdBefore")] = (
        None
    )
    types: typing.Optional[typing.List[FeedbackType]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .feedback_analytics_request import FeedbackAnalyticsRequest
import typing_extensions
import typing
from .time_interval import TimeInterval
from ...core.serialization import FieldMetadata
import pydantic
from .feedback_group_by import FeedbackGroupBy
from .feedback_column_definition import FeedbackColumnDefinition
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class FeedbackTableRequest(FeedbackAnalyticsRequest):
    """
    Examples
    --------
    from mavenagi.analytics import (
        FeedbackColumnDefinition,
        FeedbackGroupBy,
        FeedbackMetric_Count,
        FeedbackTableRequest,
    )
    from mavenagi.conversation import FeedbackFilter

    FeedbackTableRequest(
        feedback_filter=FeedbackFilter(
            types=["THUMBS_UP", "INSERT"],
        ),
        field_groupings=[
            FeedbackGroupBy(
                field="CreatedBy",
            )
        ],
        column_definitions=[
            FeedbackColumnDefinition(
                header="feedback_count",
                metric=FeedbackMetric_Count(),
            )
        ],
    )
    """

    time_grouping: typing_extensions.Annotated[typing.Optional[TimeInterval], FieldMetadata(alias="timeGrouping")] = (
        pydantic.Field(default=None)
    )
    """
    Defines the time interval for grouping data. If specified, data is grouped accordingly based on the time they were created.
     Example: If set to "DAY," data will be aggregated by day.
    """

    field_groupings: typing_extensions.Annotated[
        typing.List[FeedbackGroupBy], FieldMetadata(alias="fieldGroupings")
    ] = pydantic.Field()
    """
    Specifies the fields by which data should be grouped. Each unique combination forms a row.
    If multiple fields are provided, the result is grouped by their unique value combinations.
    If empty, all data is aggregated into a single row.
    """

    column_definitions: typing_extensions.Annotated[
        typing.List[FeedbackColumnDefinition], FieldMetadata(alias="columnDefinitions")
    ] = pydantic.Field()
    """
    Specifies the metrics to be displayed as columns.
    Column headers act as keys, with computed metric values as their mapped values.
    There needs to be at least one column definition in the table request.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
from .event_trigger_type import EventTriggerType
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class EventTriggerBase(UniversalBaseModel):
    description: str = pydantic.Field()
    """
    The description of what the event trigger does, shown in the Maven Dashboard
    """

    type: EventTriggerType = pydantic.Field()
    """
    The type of event trigger this app wishes to handle.
    
    Conversation triggers fire when a conversation is created, after each additional message, and upon deletion events.
    There is a small delay before trigger execution to allow time for conversation analysis to complete.
    
    Feedback can not be modified, so the feedback trigger fires immediately after feedback is created.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .conversation_analytics_request import ConversationAnalyticsRequest
import typing_extensions
from .conversation_group_by import ConversationGroupBy
from ...core.serialization import FieldMetadata
import pydantic
from .conversation_metric import ConversationMetric
import typing
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class BarChartRequest(ConversationAnalyticsRequest):
    bar_definition: typing_extensions.Annotated[ConversationGroupBy, FieldMetadata(alias="barDefinition")] = (
        pydantic.Field()
    )
    """
    Determines how data is grouped along the x-axis. Each unique value forms a separate bar.
    The name of the bar is derived from the grouping field's value or range.
    """

    metric: ConversationMetric = pydantic.Field()
    """
    Metric defining the y-axis values for the bar chart.
    """

    vertical_grouping: typing_extensions.Annotated[
        typing.Optional[ConversationGroupBy], FieldMetadata(alias="verticalGrouping")
    ] = pydantic.Field(default=None)
    """
    Optionally defines vertical grouping within each bar, producing multiple series.
    If omitted, a single series is generated.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
import httpx
from .http_client import HttpClient
from .http_client import AsyncHttpClient


class BaseClientWrapper:
    def __init__(
        self,
        *,
        organization_id: str,
        agent_id: str,
        app_id: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = None,
        app_secret: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = None,
        base_url: str,
        timeout: typing.Optional[float] = None,
    ):
        self._organization_id = organization_id
        self._agent_id = agent_id
        self._app_id = app_id
        self._app_secret = app_secret
        self._base_url = base_url
        self._timeout = timeout

    def get_headers(self) -> typing.Dict[str, str]:
        headers: typing.Dict[str, str] = {
            "X-Fern-Language": "Python",
            "X-Fern-SDK-Name": "mavenagi",
            "X-Fern-SDK-Version": "1.0.7",
        }
        app_id = self._get_app_id()
        app_secret = self._get_app_secret()
        if app_id is not None and app_secret is not None:
            headers["Authorization"] = httpx.BasicAuth(app_id, app_secret)._auth_header
        headers["X-Organization-Id"] = self._organization_id
        headers["X-Agent-Id"] = self._agent_id
        return headers

    def _get_app_id(self) -> typing.Optional[str]:
        if isinstance(self._app_id, str) or self._app_id is None:
            return self._app_id
        else:
            return self._app_id()

    def _get_app_secret(self) -> typing.Optional[str]:
        if isinstance(self._app_secret, str) or self._app_secret is None:
            return self._app_secret
        else:
            return self._app_secret()

    def get_base_url(self) -> str:
        return self._base_url

    def get_timeout(self) -> typing.Optional[float]:
        return self._timeout


class SyncClientWrapper(BaseClientWrapper):
    def __init__(
        self,
        *,
        organization_id: str,
        agent_id: str,
        app_id: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = None,
        app_secret: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = None,
        base_url: str,
        timeout: typing.Optional[float] = None,
        httpx_client: httpx.Client,
    ):
        super().__init__(
            organization_id=organization_id,
            agent_id=agent_id,
            app_id=app_id,
            app_secret=app_secret,
            base_url=base_url,
            timeout=timeout,
        )
        self.httpx_client = HttpClient(
            httpx_client=httpx_client,
            base_headers=self.get_headers,
            base_timeout=self.get_timeout,
            base_url=self.get_base_url,
        )


class AsyncClientWrapper(BaseClientWrapper):
    def __init__(
        self,
        *,
        organization_id: str,
        agent_id: str,
        app_id: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = None,
        app_secret: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = None,
        base_url: str,
        timeout: typing.Optional[float] = None,
        httpx_client: httpx.AsyncClient,
    ):
        super().__init__(
            organization_id=organization_id,
            agent_id=agent_id,
            app_id=app_id,
            app_secret=app_secret,
            base_url=base_url,
            timeout=timeout,
        )
        self.httpx_client = AsyncHttpClient(
            httpx_client=httpx_client,
            base_headers=self.get_headers,
            base_timeout=self.get_timeout,
            base_url=self.get_base_url,
        )



# This file was auto-generated by Fern from our API Definition.

import enum


class MavenAGIEnvironment(enum.Enum):
    PRODUCTION = "https://www.mavenagi-apis.com"
    STAGING = "https://www.staging.mavenagi-apis.com"



# This file was auto-generated by Fern from our API Definition.

from .conversation_analytics_request import ConversationAnalyticsRequest
import typing_extensions
from .conversation_group_by import ConversationGroupBy
from ...core.serialization import FieldMetadata
import pydantic
from .conversation_metric import ConversationMetric
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class PieChartRequest(ConversationAnalyticsRequest):
    group_by: typing_extensions.Annotated[ConversationGroupBy, FieldMetadata(alias="groupBy")] = pydantic.Field()
    """
    Field used to group data into slices for the pie chart.
    """

    metric: ConversationMetric = pydantic.Field()
    """
    Metric defining the value for each pie slice, stored in the y-axis value.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing_extensions
import typing
from ...conversation.types.conversation_filter import ConversationFilter
from ...core.serialization import FieldMetadata
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class ConversationAnalyticsRequest(UniversalBaseModel):
    conversation_filter: typing_extensions.Annotated[
        typing.Optional[ConversationFilter], FieldMetadata(alias="conversationFilter")
    ] = pydantic.Field(default=None)
    """
    Optional filter applied to refine the conversation data before processing.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .table_response_base import TableResponseBase
import typing
from .feedback_row import FeedbackRow
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class FeedbackTableResponse(TableResponseBase):
    """
    Examples
    --------
    from mavenagi.analytics import (
        CellData_Double,
        FeedbackRow,
        FeedbackTableResponse,
        FieldValue_String,
    )

    FeedbackTableResponse(
        headers=["feedback_count", "unique_users"],
        rows=[
            FeedbackRow(
                identifier={"CreatedBy": FieldValue_String(value="John Doe")},
                data={"feedback_count": CellData_Double(value=5.0)},
            ),
            FeedbackRow(
                identifier={"CreatedBy": FieldValue_String(value="Jane Smith")},
                data={"feedback_count": CellData_Double(value=3.0)},
            ),
        ],
    )
    """

    rows: typing.List[FeedbackRow] = pydantic.Field()
    """
    The dataset rows, where each row represents a unique combination of grouping field values. 
    The identifier map contains grouping field names mapped to their respective values.
    The data map contains column headers mapped to their respective metric values.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing_extensions
from .chart_spec_schema import ChartSpecSchema
from ...core.serialization import FieldMetadata
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class BotChartResponse(UniversalBaseModel):
    """
    Will only be provided if the responseConfig contains the charts_highcharts_ts capability.
    """

    label: str = pydantic.Field()
    """
    The label of the chart
    """

    spec_schema: typing_extensions.Annotated[ChartSpecSchema, FieldMetadata(alias="specSchema")]
    spec: str = pydantic.Field()
    """
    The spec string for the chart. For HIGHCHARTS_TS charts, the spec is the json object that represents the chart options.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing
from .action_parameter_type import ActionParameterType
import typing_extensions
from .action_enum_option import ActionEnumOption
from ...core.serialization import FieldMetadata
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class ActionParameter(UniversalBaseModel):
    id: str = pydantic.Field()
    """
    The ID field will be used when parameters are supplied to `executeAction`.
    """

    label: str = pydantic.Field()
    """
    The title of the field that will be shown on action forms.
    """

    description: str = pydantic.Field()
    """
    A longer description of the field which will be shown in smaller text near the label on action forms.
    """

    required: bool = pydantic.Field()
    """
    Whether the field is required for action execution.
    """

    type: typing.Optional[ActionParameterType] = pydantic.Field(default=None)
    """
    The parameter type. Values provided to `executeAction` will conform to this type. Defaults to `STRING`.
    """

    enum_options: typing_extensions.Annotated[
        typing.Optional[typing.List[ActionEnumOption]], FieldMetadata(alias="enumOptions")
    ] = pydantic.Field(default=None)
    """
    Restricts the action parameter to only the options in this list. Valid for type `STRING`, `BOOLEAN`, and `NUMBER`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .translation_request import TranslationRequest
from .translation_response import TranslationResponse

__all__ = ["TranslationRequest", "TranslationResponse"]



# This file was auto-generated by Fern from our API Definition.

from .conversation_message_base import ConversationMessageBase
import typing_extensions
from .entity_id import EntityId
from ...core.serialization import FieldMetadata
import pydantic
from .bot_conversation_message_type import BotConversationMessageType
import typing
from .bot_response import BotResponse
from .bot_response_metadata import BotResponseMetadata
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class BotMessage(ConversationMessageBase):
    conversation_message_id: typing_extensions.Annotated[EntityId, FieldMetadata(alias="conversationMessageId")] = (
        pydantic.Field()
    )
    """
    The ID that uniquely identifies this message within the conversation
    """

    bot_message_type: typing_extensions.Annotated[BotConversationMessageType, FieldMetadata(alias="botMessageType")]
    responses: typing.List[BotResponse]
    metadata: BotResponseMetadata

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing_extensions
from ...core.serialization import FieldMetadata
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing
import pydantic


class EntityIdFilter(UniversalBaseModel):
    reference_id: typing_extensions.Annotated[str, FieldMetadata(alias="referenceId")]
    app_id: typing_extensions.Annotated[str, FieldMetadata(alias="appId")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ...core.pydantic_utilities import UniversalBaseModel
import typing_extensions
import typing
from ...core.serialization import FieldMetadata
from .precondition_operator import PreconditionOperator
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic
from .response_length import ResponseLength


class ConversationPrecondition_Tags(UniversalBaseModel):
    conversation_precondition_type: typing_extensions.Annotated[
        typing.Literal["tags"], FieldMetadata(alias="conversationPreconditionType")
    ] = "tags"
    tags: typing.Set[str]
    operator: typing.Optional[PreconditionOperator] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ConversationPrecondition_Metadata(UniversalBaseModel):
    conversation_precondition_type: typing_extensions.Annotated[
        typing.Literal["metadata"], FieldMetadata(alias="conversationPreconditionType")
    ] = "metadata"
    key: str
    value: typing.Optional[str] = None
    operator: typing.Optional[PreconditionOperator] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ConversationPrecondition_ActionExecuted(UniversalBaseModel):
    conversation_precondition_type: typing_extensions.Annotated[
        typing.Literal["actionExecuted"], FieldMetadata(alias="conversationPreconditionType")
    ] = "actionExecuted"
    action_id: typing_extensions.Annotated[str, FieldMetadata(alias="actionId")]
    app_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="appId")] = None
    operator: typing.Optional[PreconditionOperator] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ConversationPrecondition_ResponseConfig(UniversalBaseModel):
    conversation_precondition_type: typing_extensions.Annotated[
        typing.Literal["responseConfig"], FieldMetadata(alias="conversationPreconditionType")
    ] = "responseConfig"
    use_markdown: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="useMarkdown")] = None
    use_forms: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="useForms")] = None
    use_images: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="useImages")] = None
    is_copilot: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="isCopilot")] = None
    response_length: typing_extensions.Annotated[
        typing.Optional[ResponseLength], FieldMetadata(alias="responseLength")
    ] = None
    operator: typing.Optional[PreconditionOperator] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


ConversationPrecondition = typing.Union[
    ConversationPrecondition_Tags,
    ConversationPrecondition_Metadata,
    ConversationPrecondition_ActionExecuted,
    ConversationPrecondition_ResponseConfig,
]



# This file was auto-generated by Fern from our API Definition.

from .entity_id_without_agent import EntityIdWithoutAgent
import typing_extensions
from ...core.serialization import FieldMetadata
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class EntityId(EntityIdWithoutAgent):
    """
    A fully specified object ID, unique across the entire system.
    """

    organization_id: typing_extensions.Annotated[str, FieldMetadata(alias="organizationId")] = pydantic.Field()
    """
    The ID of the organization that this object belongs to
    """

    agent_id: typing_extensions.Annotated[str, FieldMetadata(alias="agentId")] = pydantic.Field()
    """
    The ID of the agent that this object belongs to
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .conversation_analytics_request import ConversationAnalyticsRequest
import typing_extensions
from .time_interval import TimeInterval
from ...core.serialization import FieldMetadata
import pydantic
import typing
from .conversation_group_by import ConversationGroupBy
from .conversation_metric import ConversationMetric
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class DateHistogramRequest(ConversationAnalyticsRequest):
    time_interval: typing_extensions.Annotated[TimeInterval, FieldMetadata(alias="timeInterval")] = pydantic.Field()
    """
    Time-based grouping interval (e.g., HOUR, DAY, WEEK) for the date histogram.
    """

    group_by: typing_extensions.Annotated[typing.Optional[ConversationGroupBy], FieldMetadata(alias="groupBy")] = (
        pydantic.Field(default=None)
    )
    """
    Groups data before applying calculations, forming a separate time series for each group.
    """

    metric: ConversationMetric = pydantic.Field()
    """
    Defines the y-axis values for the date histogram.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from . import actions, analytics, app_settings, commons, conversation, knowledge, translations, triggers, users
from .actions import ActionRequest
from .analytics import (
    BarChartRequest,
    BarChartResponse,
    CellData,
    CellData_Double,
    CellData_Long,
    CellData_Millisecond,
    CellData_String,
    ChartResponse,
    ChartResponse_BarChart,
    ChartResponse_DateHistogram,
    ChartResponse_PieChart,
    ColumnDefinitionBase,
    ConversationAnalyticsRequest,
    ConversationAverage,
    ConversationBasicMetric,
    ConversationChartRequest,
    ConversationChartRequest_BarChart,
    ConversationChartRequest_DateHistogram,
    ConversationChartRequest_PieChart,
    ConversationColumnDefinition,
    ConversationCount,
    ConversationDistinctCount,
    ConversationGroupBy,
    ConversationMax,
    ConversationMedian,
    ConversationMetric,
    ConversationMetric_Average,
    ConversationMetric_Count,
    ConversationMetric_DistinctCount,
    ConversationMetric_Max,
    ConversationMetric_Median,
    ConversationMetric_Min,
    ConversationMetric_Percentile,
    ConversationMetric_Sum,
    ConversationMin,
    ConversationNumericMetric,
    ConversationPercentile,
    ConversationRow,
    ConversationSum,
    ConversationTableRequest,
    ConversationTableResponse,
    DateHistogramRequest,
    DateHistogramResponse,
    FeedbackAnalyticsRequest,
    FeedbackColumnDefinition,
    FeedbackCount,
    FeedbackDistinctCount,
    FeedbackGroupBy,
    FeedbackMetric,
    FeedbackMetric_Count,
    FeedbackMetric_DistinctCount,
    FeedbackRow,
    FeedbackTableRequest,
    FeedbackTableResponse,
    FieldValue,
    FieldValue_Boolean,
    FieldValue_DateTime,
    FieldValue_Double,
    FieldValue_EntityId,
    FieldValue_Long,
    FieldValue_Range,
    FieldValue_String,
    GroupByBase,
    LabeledPoint,
    PieChartRequest,
    PieChartResponse,
    Range,
    RowBase,
    Series,
    TableResponseBase,
    TimeDataPoint,
    TimeInterval,
    TimeSeries,
)
from .client import AsyncMavenAGI, MavenAGI
from .commons import (
    ActionBase,
    ActionEnumOption,
    ActionFormField,
    ActionParameter,
    ActionParameterType,
    ActionResponse,
    AppUser,
    AppUserIdentifier,
    AppUserIdentifyingPropertyType,
    AppUserRequest,
    AppUserResponse,
    BadRequestError,
    BotActionFormResponse,
    BotChartResponse,
    BotConversationMessageType,
    BotMessage,
    BotResponse,
    BotResponseMetadata,
    BotResponse_ActionForm,
    BotResponse_Chart,
    BotResponse_Text,
    BotTextResponse,
    Capability,
    ChartSpecSchema,
    ConversationAnalysis,
    ConversationBase,
    ConversationExecutedActionPrecondition,
    ConversationMessageBase,
    ConversationMessageResponse,
    ConversationMessageResponse_Bot,
    ConversationMessageResponse_User,
    ConversationPrecondition,
    ConversationPrecondition_ActionExecuted,
    ConversationPrecondition_Metadata,
    ConversationPrecondition_ResponseConfig,
    ConversationPrecondition_Tags,
    ConversationResponse,
    ConversationSummary,
    EntityId,
    EntityIdBase,
    EntityIdFilter,
    EntityIdWithoutAgent,
    EntityType,
    ErrorMessage,
    EventTriggerBase,
    EventTriggerResponse,
    EventTriggerType,
    Feedback,
    FeedbackBase,
    FeedbackType,
    MetadataPrecondition,
    NotFoundError,
    Precondition,
    PreconditionBase,
    PreconditionGroup,
    PreconditionGroupOperator,
    PreconditionOperator,
    Precondition_Conversation,
    Precondition_Group,
    Precondition_User,
    ResponseConfig,
    ResponseConfigPrecondition,
    ResponseLength,
    Sentiment,
    ServerError,
    Source,
    TagsPrecondition,
    UserConversationMessageType,
    UserData,
    UserMessage,
    UserMessageAttachment,
    UserMessageBase,
    VisibilityType,
)
from .conversation import (
    AskRequest,
    AskStreamActionEvent,
    AskStreamChartEvent,
    AskStreamEndEvent,
    AskStreamMetadataEvent,
    AskStreamStartEvent,
    AskStreamTextEvent,
    Attachment,
    CategorizationResponse,
    ConversationField,
    ConversationFilter,
    ConversationMessageRequest,
    ConversationRequest,
    FeedbackField,
    FeedbackFilter,
    FeedbackRequest,
    GenerateMavenSuggestionsRequest,
    NumericConversationField,
    Quality,
    QualityReason,
    StreamResponse,
    StreamResponse_Action,
    StreamResponse_Chart,
    StreamResponse_End,
    StreamResponse_Metadata,
    StreamResponse_Start,
    StreamResponse_Text,
    SubmitActionFormRequest,
)
from .environment import MavenAGIEnvironment
from .knowledge import (
    BaseKnowledgeDocument,
    KnowledgeBaseProperties,
    KnowledgeBaseRequest,
    KnowledgeBaseResponse,
    KnowledgeBaseType,
    KnowledgeBaseVersion,
    KnowledgeBaseVersionType,
    KnowledgeDocumentContentType,
    KnowledgeDocumentRequest,
    KnowledgeDocumentResponse,
)
from .translations import TranslationRequest, TranslationResponse
from .triggers import EventTriggerRequest
from .version import __version__

__all__ = [
    "ActionBase",
    "ActionEnumOption",
    "ActionFormField",
    "ActionParameter",
    "ActionParameterType",
    "ActionRequest",
    "ActionResponse",
    "AppUser",
    "AppUserIdentifier",
    "AppUserIdentifyingPropertyType",
    "AppUserRequest",
    "AppUserResponse",
    "AskRequest",
    "AskStreamActionEvent",
    "AskStreamChartEvent",
    "AskStreamEndEvent",
    "AskStreamMetadataEvent",
    "AskStreamStartEvent",
    "AskStreamTextEvent",
    "AsyncMavenAGI",
    "Attachment",
    "BadRequestError",
    "BarChartRequest",
    "BarChartResponse",
    "BaseKnowledgeDocument",
    "BotActionFormResponse",
    "BotChartResponse",
    "BotConversationMessageType",
    "BotMessage",
    "BotResponse",
    "BotResponseMetadata",
    "BotResponse_ActionForm",
    "BotResponse_Chart",
    "BotResponse_Text",
    "BotTextResponse",
    "Capability",
    "CategorizationResponse",
    "CellData",
    "CellData_Double",
    "CellData_Long",
    "CellData_Millisecond",
    "CellData_String",
    "ChartResponse",
    "ChartResponse_BarChart",
    "ChartResponse_DateHistogram",
    "ChartResponse_PieChart",
    "ChartSpecSchema",
    "ColumnDefinitionBase",
    "ConversationAnalysis",
    "ConversationAnalyticsRequest",
    "ConversationAverage",
    "ConversationBase",
    "ConversationBasicMetric",
    "ConversationChartRequest",
    "ConversationChartRequest_BarChart",
    "ConversationChartRequest_DateHistogram",
    "ConversationChartRequest_PieChart",
    "ConversationColumnDefinition",
    "ConversationCount",
    "ConversationDistinctCount",
    "ConversationExecutedActionPrecondition",
    "ConversationField",
    "ConversationFilter",
    "ConversationGroupBy",
    "ConversationMax",
    "ConversationMedian",
    "ConversationMessageBase",
    "ConversationMessageRequest",
    "ConversationMessageResponse",
    "ConversationMessageResponse_Bot",
    "ConversationMessageResponse_User",
    "ConversationMetric",
    "ConversationMetric_Average",
    "ConversationMetric_Count",
    "ConversationMetric_DistinctCount",
    "ConversationMetric_Max",
    "ConversationMetric_Median",
    "ConversationMetric_Min",
    "ConversationMetric_Percentile",
    "ConversationMetric_Sum",
    "ConversationMin",
    "ConversationNumericMetric",
    "ConversationPercentile",
    "ConversationPrecondition",
    "ConversationPrecondition_ActionExecuted",
    "ConversationPrecondition_Metadata",
    "ConversationPrecondition_ResponseConfig",
    "ConversationPrecondition_Tags",
    "ConversationRequest",
    "ConversationResponse",
    "ConversationRow",
    "ConversationSum",
    "ConversationSummary",
    "ConversationTableRequest",
    "ConversationTableResponse",
    "DateHistogramRequest",
    "DateHistogramResponse",
    "EntityId",
    "EntityIdBase",
    "EntityIdFilter",
    "EntityIdWithoutAgent",
    "EntityType",
    "ErrorMessage",
    "EventTriggerBase",
    "EventTriggerRequest",
    "EventTriggerResponse",
    "EventTriggerType",
    "Feedback",
    "FeedbackAnalyticsRequest",
    "FeedbackBase",
    "FeedbackColumnDefinition",
    "FeedbackCount",
    "FeedbackDistinctCount",
    "FeedbackField",
    "FeedbackFilter",
    "FeedbackGroupBy",
    "FeedbackMetric",
    "FeedbackMetric_Count",
    "FeedbackMetric_DistinctCount",
    "FeedbackRequest",
    "FeedbackRow",
    "FeedbackTableRequest",
    "FeedbackTableResponse",
    "FeedbackType",
    "FieldValue",
    "FieldValue_Boolean",
    "FieldValue_DateTime",
    "FieldValue_Double",
    "FieldValue_EntityId",
    "FieldValue_Long",
    "FieldValue_Range",
    "FieldValue_String",
    "GenerateMavenSuggestionsRequest",
    "GroupByBase",
    "KnowledgeBaseProperties",
    "KnowledgeBaseRequest",
    "KnowledgeBaseResponse",
    "KnowledgeBaseType",
    "KnowledgeBaseVersion",
    "KnowledgeBaseVersionType",
    "KnowledgeDocumentContentType",
    "KnowledgeDocumentRequest",
    "KnowledgeDocumentResponse",
    "LabeledPoint",
    "MavenAGI",
    "MavenAGIEnvironment",
    "MetadataPrecondition",
    "NotFoundError",
    "NumericConversationField",
    "PieChartRequest",
    "PieChartResponse",
    "Precondition",
    "PreconditionBase",
    "PreconditionGroup",
    "PreconditionGroupOperator",
    "PreconditionOperator",
    "Precondition_Conversation",
    "Precondition_Group",
    "Precondition_User",
    "Quality",
    "QualityReason",
    "Range",
    "ResponseConfig",
    "ResponseConfigPrecondition",
    "ResponseLength",
    "RowBase",
    "Sentiment",
    "Series",
    "ServerError",
    "Source",
    "StreamResponse",
    "StreamResponse_Action",
    "StreamResponse_Chart",
    "StreamResponse_End",
    "StreamResponse_Metadata",
    "StreamResponse_Start",
    "StreamResponse_Text",
    "SubmitActionFormRequest",
    "TableResponseBase",
    "TagsPrecondition",
    "TimeDataPoint",
    "TimeInterval",
    "TimeSeries",
    "TranslationRequest",
    "TranslationResponse",
    "UserConversationMessageType",
    "UserData",
    "UserMessage",
    "UserMessageAttachment",
    "UserMessageBase",
    "VisibilityType",
    "__version__",
    "actions",
    "analytics",
    "app_settings",
    "commons",
    "conversation",
    "knowledge",
    "translations",
    "triggers",
    "users",
]



# This file was auto-generated by Fern from our API Definition.

import typing

ConversationField = typing.Union[
    typing.Literal[
        "Category",
        "FirstResponseTime",
        "HandleTime",
        "HumanAgents",
        "App",
        "Sentiment",
        "QualityReason",
        "ResolutionStatus",
        "Quality",
        "Users",
        "ResponseLength",
        "ThumbsUpCount",
        "ThumbsDownCount",
        "Tags",
        "UserMessageCount",
        "Languages",
        "Actions",
        "IncompleteActions",
        "Sources",
        "CreatedAt",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ...core.pydantic_utilities import UniversalBaseModel
import typing
from .series import Series
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic
import typing_extensions
from .time_series import TimeSeries
from ...core.serialization import FieldMetadata


class ChartResponse_PieChart(UniversalBaseModel):
    """
    Examples
    --------
    from mavenagi.analytics import ChartResponse_PieChart, LabeledPoint, Series

    ChartResponse_PieChart(
        series=Series(
            name="Series",
            data=[
                LabeledPoint(
                    label="Sales",
                    y=5.0,
                ),
                LabeledPoint(
                    label="Support",
                    y=10.0,
                ),
            ],
        ),
    )
    """

    type: typing.Literal["pieChart"] = "pieChart"
    series: Series

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ChartResponse_DateHistogram(UniversalBaseModel):
    """
    Examples
    --------
    from mavenagi.analytics import ChartResponse_PieChart, LabeledPoint, Series

    ChartResponse_PieChart(
        series=Series(
            name="Series",
            data=[
                LabeledPoint(
                    label="Sales",
                    y=5.0,
                ),
                LabeledPoint(
                    label="Support",
                    y=10.0,
                ),
            ],
        ),
    )
    """

    type: typing.Literal["dateHistogram"] = "dateHistogram"
    time_series: typing_extensions.Annotated[typing.List[TimeSeries], FieldMetadata(alias="timeSeries")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ChartResponse_BarChart(UniversalBaseModel):
    """
    Examples
    --------
    from mavenagi.analytics import ChartResponse_PieChart, LabeledPoint, Series

    ChartResponse_PieChart(
        series=Series(
            name="Series",
            data=[
                LabeledPoint(
                    label="Sales",
                    y=5.0,
                ),
                LabeledPoint(
                    label="Support",
                    y=10.0,
                ),
            ],
        ),
    )
    """

    type: typing.Literal["barChart"] = "barChart"
    series: typing.List[Series]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


"""
from mavenagi.analytics import ChartResponse_PieChart, LabeledPoint, Series

ChartResponse_PieChart(
    series=Series(
        name="Series",
        data=[
            LabeledPoint(
                label="Sales",
                y=5.0,
            ),
            LabeledPoint(
                label="Support",
                y=10.0,
            ),
        ],
    ),
)
"""
ChartResponse = typing.Union[ChartResponse_PieChart, ChartResponse_DateHistogram, ChartResponse_BarChart]



# This file was auto-generated by Fern from our API Definition.

from .column_definition_base import ColumnDefinitionBase
from .conversation_metric import ConversationMetric
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class ConversationColumnDefinition(ColumnDefinitionBase):
    metric: ConversationMetric = pydantic.Field()
    """
    The metric calculated for this column, stored in the row data under the specified header.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .base_knowledge_document import BaseKnowledgeDocument
import typing_extensions
from ...commons.types.entity_id import EntityId
from ...core.serialization import FieldMetadata
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class KnowledgeDocumentResponse(BaseKnowledgeDocument):
    """
    Examples
    --------
    from mavenagi.commons import EntityId
    from mavenagi.knowledge import KnowledgeDocumentResponse

    KnowledgeDocumentResponse(
        knowledge_document_id=EntityId(
            reference_id="getting-started",
            app_id="readme",
            organization_id="acme",
            agent_id="support",
            type="KNOWLEDGE_DOCUMENT",
        ),
        content="## Getting started This is a getting started guide for the help center.",
        title="Getting started",
    )
    """

    knowledge_document_id: typing_extensions.Annotated[EntityId, FieldMetadata(alias="knowledgeDocumentId")] = (
        pydantic.Field()
    )
    """
    ID that uniquely identifies this knowledge document within its knowledge base
    """

    content: str = pydantic.Field()
    """
    The content of the document in markdown format. Not shown directly to users.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing_extensions
from ...core.serialization import FieldMetadata
import typing
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class SubmitActionFormRequest(UniversalBaseModel):
    action_form_id: typing_extensions.Annotated[str, FieldMetadata(alias="actionFormId")]
    parameters: typing.Dict[str, typing.Optional[typing.Any]] = pydantic.Field()
    """
    Map of parameter IDs to values provided by the user. All required action fields must be provided.
    """

    transient_data: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, str]], FieldMetadata(alias="transientData")
    ] = pydantic.Field(default=None)
    """
    Transient data which the Maven platform will not persist. This data will only be forwarded to actions taken. For example, one may put in user tokens as transient data.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .group_by_base import GroupByBase
from ...conversation.types.conversation_field import ConversationField
import pydantic
import typing
from .range import Range
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class ConversationGroupBy(GroupByBase):
    field: ConversationField = pydantic.Field()
    """
    Field used for data grouping.
    """

    ranges: typing.Optional[typing.List[Range]] = pydantic.Field(default=None)
    """
    Numeric ranges for grouping data into predefined buckets. Applies only to numeric fields.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .user_message_base import UserMessageBase
import typing_extensions
from .entity_id import EntityId
from ...core.serialization import FieldMetadata
import pydantic
import typing
from .user_message_attachment import UserMessageAttachment
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class UserMessage(UserMessageBase):
    conversation_message_id: typing_extensions.Annotated[EntityId, FieldMetadata(alias="conversationMessageId")] = (
        pydantic.Field()
    )
    """
    The ID that uniquely identifies this message within the conversation
    """

    language: typing.Optional[str] = pydantic.Field(default=None)
    """
    The language of the message in ISO 639-1 code format
    """

    attachments: typing.List[UserMessageAttachment] = pydantic.Field()
    """
    The attachments associated with the message
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .conversation_numeric_metric import ConversationNumericMetric
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class ConversationPercentile(ConversationNumericMetric):
    """
    Calculates specified percentile for a numeric field.
    """

    percentile: float = pydantic.Field()
    """
    The percentile to calculate. Example: 25 computes the 25th percentile.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .base_knowledge_document import BaseKnowledgeDocument
from .knowledge_base_properties import KnowledgeBaseProperties
from .knowledge_base_request import KnowledgeBaseRequest
from .knowledge_base_response import KnowledgeBaseResponse
from .knowledge_base_type import KnowledgeBaseType
from .knowledge_base_version import KnowledgeBaseVersion
from .knowledge_base_version_type import KnowledgeBaseVersionType
from .knowledge_document_content_type import KnowledgeDocumentContentType
from .knowledge_document_request import KnowledgeDocumentRequest
from .knowledge_document_response import KnowledgeDocumentResponse

__all__ = [
    "BaseKnowledgeDocument",
    "KnowledgeBaseProperties",
    "KnowledgeBaseRequest",
    "KnowledgeBaseResponse",
    "KnowledgeBaseType",
    "KnowledgeBaseVersion",
    "KnowledgeBaseVersionType",
    "KnowledgeDocumentContentType",
    "KnowledgeDocumentRequest",
    "KnowledgeDocumentResponse",
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ...core.pydantic_utilities import UniversalBaseModel
from .precondition_group_operator import PreconditionGroupOperator
import typing
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic
from ...core.pydantic_utilities import update_forward_refs


class PreconditionGroup(UniversalBaseModel):
    operator: PreconditionGroupOperator
    preconditions: typing.List["Precondition"]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


from .precondition import Precondition  # noqa: E402

update_forward_refs(PreconditionGroup)



# This file was auto-generated by Fern from our API Definition.

from .types import (
    BarChartRequest,
    BarChartResponse,
    CellData,
    CellData_Double,
    CellData_Long,
    CellData_Millisecond,
    CellData_String,
    ChartResponse,
    ChartResponse_BarChart,
    ChartResponse_DateHistogram,
    ChartResponse_PieChart,
    ColumnDefinitionBase,
    ConversationAnalyticsRequest,
    ConversationAverage,
    ConversationBasicMetric,
    ConversationChartRequest,
    ConversationChartRequest_BarChart,
    ConversationChartRequest_DateHistogram,
    ConversationChartRequest_PieChart,
    ConversationColumnDefinition,
    ConversationCount,
    ConversationDistinctCount,
    ConversationGroupBy,
    ConversationMax,
    ConversationMedian,
    ConversationMetric,
    ConversationMetric_Average,
    ConversationMetric_Count,
    ConversationMetric_DistinctCount,
    ConversationMetric_Max,
    ConversationMetric_Median,
    ConversationMetric_Min,
    ConversationMetric_Percentile,
    ConversationMetric_Sum,
    ConversationMin,
    ConversationNumericMetric,
    ConversationPercentile,
    ConversationRow,
    ConversationSum,
    ConversationTableRequest,
    ConversationTableResponse,
    DateHistogramRequest,
    DateHistogramResponse,
    FeedbackAnalyticsRequest,
    FeedbackColumnDefinition,
    FeedbackCount,
    FeedbackDistinctCount,
    FeedbackGroupBy,
    FeedbackMetric,
    FeedbackMetric_Count,
    FeedbackMetric_DistinctCount,
    FeedbackRow,
    FeedbackTableRequest,
    FeedbackTableResponse,
    FieldValue,
    FieldValue_Boolean,
    FieldValue_DateTime,
    FieldValue_Double,
    FieldValue_EntityId,
    FieldValue_Long,
    FieldValue_Range,
    FieldValue_String,
    GroupByBase,
    LabeledPoint,
    PieChartRequest,
    PieChartResponse,
    Range,
    RowBase,
    Series,
    TableResponseBase,
    TimeDataPoint,
    TimeInterval,
    TimeSeries,
)

__all__ = [
    "BarChartRequest",
    "BarChartResponse",
    "CellData",
    "CellData_Double",
    "CellData_Long",
    "CellData_Millisecond",
    "CellData_String",
    "ChartResponse",
    "ChartResponse_BarChart",
    "ChartResponse_DateHistogram",
    "ChartResponse_PieChart",
    "ColumnDefinitionBase",
    "ConversationAnalyticsRequest",
    "ConversationAverage",
    "ConversationBasicMetric",
    "ConversationChartRequest",
    "ConversationChartRequest_BarChart",
    "ConversationChartRequest_DateHistogram",
    "ConversationChartRequest_PieChart",
    "ConversationColumnDefinition",
    "ConversationCount",
    "ConversationDistinctCount",
    "ConversationGroupBy",
    "ConversationMax",
    "ConversationMedian",
    "ConversationMetric",
    "ConversationMetric_Average",
    "ConversationMetric_Count",
    "ConversationMetric_DistinctCount",
    "ConversationMetric_Max",
    "ConversationMetric_Median",
    "ConversationMetric_Min",
    "ConversationMetric_Percentile",
    "ConversationMetric_Sum",
    "ConversationMin",
    "ConversationNumericMetric",
    "ConversationPercentile",
    "ConversationRow",
    "ConversationSum",
    "ConversationTableRequest",
    "ConversationTableResponse",
    "DateHistogramRequest",
    "DateHistogramResponse",
    "FeedbackAnalyticsRequest",
    "FeedbackColumnDefinition",
    "FeedbackCount",
    "FeedbackDistinctCount",
    "FeedbackGroupBy",
    "FeedbackMetric",
    "FeedbackMetric_Count",
    "FeedbackMetric_DistinctCount",
    "FeedbackRow",
    "FeedbackTableRequest",
    "FeedbackTableResponse",
    "FieldValue",
    "FieldValue_Boolean",
    "FieldValue_DateTime",
    "FieldValue_Double",
    "FieldValue_EntityId",
    "FieldValue_Long",
    "FieldValue_Range",
    "FieldValue_String",
    "GroupByBase",
    "LabeledPoint",
    "PieChartRequest",
    "PieChartResponse",
    "Range",
    "RowBase",
    "Series",
    "TableResponseBase",
    "TimeDataPoint",
    "TimeInterval",
    "TimeSeries",
]



# This file was auto-generated by Fern from our API Definition.

import typing
from .environment import MavenAGIEnvironment
import os
import httpx
from .core.client_wrapper import SyncClientWrapper
from .actions.client import ActionsClient
from .analytics.client import AnalyticsClient
from .app_settings.client import AppSettingsClient
from .conversation.client import ConversationClient
from .knowledge.client import KnowledgeClient
from .translations.client import TranslationsClient
from .triggers.client import TriggersClient
from .users.client import UsersClient
from .core.client_wrapper import AsyncClientWrapper
from .actions.client import AsyncActionsClient
from .analytics.client import AsyncAnalyticsClient
from .app_settings.client import AsyncAppSettingsClient
from .conversation.client import AsyncConversationClient
from .knowledge.client import AsyncKnowledgeClient
from .translations.client import AsyncTranslationsClient
from .triggers.client import AsyncTriggersClient
from .users.client import AsyncUsersClient


class MavenAGI:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : MavenAGIEnvironment
        The environment to use for requests from the client. from .environment import MavenAGIEnvironment



        Defaults to MavenAGIEnvironment.PRODUCTION



    organization_id : str
    agent_id : str
    app_id : typing.Optional[typing.Union[str, typing.Callable[[], str]]]
    app_secret : typing.Optional[typing.Union[str, typing.Callable[[], str]]]
    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.Client]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from mavenagi import MavenAGI

    client = MavenAGI(
        organization_id="YOUR_ORGANIZATION_ID",
        agent_id="YOUR_AGENT_ID",
        app_id="YOUR_APP_ID",
        app_secret="YOUR_APP_SECRET",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: MavenAGIEnvironment = MavenAGIEnvironment.PRODUCTION,
        organization_id: str,
        agent_id: str,
        app_id: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = os.getenv("MAVENAGI_APP_ID"),
        app_secret: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = os.getenv("MAVENAGI_APP_SECRET"),
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.Client] = None,
    ):
        _defaulted_timeout = timeout if timeout is not None else 60 if httpx_client is None else None
        self._client_wrapper = SyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            organization_id=organization_id,
            agent_id=agent_id,
            app_id=app_id,
            app_secret=app_secret,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.Client(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.Client(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self.actions = ActionsClient(client_wrapper=self._client_wrapper)
        self.analytics = AnalyticsClient(client_wrapper=self._client_wrapper)
        self.app_settings = AppSettingsClient(client_wrapper=self._client_wrapper)
        self.conversation = ConversationClient(client_wrapper=self._client_wrapper)
        self.knowledge = KnowledgeClient(client_wrapper=self._client_wrapper)
        self.translations = TranslationsClient(client_wrapper=self._client_wrapper)
        self.triggers = TriggersClient(client_wrapper=self._client_wrapper)
        self.users = UsersClient(client_wrapper=self._client_wrapper)


class AsyncMavenAGI:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : MavenAGIEnvironment
        The environment to use for requests from the client. from .environment import MavenAGIEnvironment



        Defaults to MavenAGIEnvironment.PRODUCTION



    organization_id : str
    agent_id : str
    app_id : typing.Optional[typing.Union[str, typing.Callable[[], str]]]
    app_secret : typing.Optional[typing.Union[str, typing.Callable[[], str]]]
    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.AsyncClient]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from mavenagi import AsyncMavenAGI

    client = AsyncMavenAGI(
        organization_id="YOUR_ORGANIZATION_ID",
        agent_id="YOUR_AGENT_ID",
        app_id="YOUR_APP_ID",
        app_secret="YOUR_APP_SECRET",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: MavenAGIEnvironment = MavenAGIEnvironment.PRODUCTION,
        organization_id: str,
        agent_id: str,
        app_id: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = os.getenv("MAVENAGI_APP_ID"),
        app_secret: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = os.getenv("MAVENAGI_APP_SECRET"),
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.AsyncClient] = None,
    ):
        _defaulted_timeout = timeout if timeout is not None else 60 if httpx_client is None else None
        self._client_wrapper = AsyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            organization_id=organization_id,
            agent_id=agent_id,
            app_id=app_id,
            app_secret=app_secret,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self.actions = AsyncActionsClient(client_wrapper=self._client_wrapper)
        self.analytics = AsyncAnalyticsClient(client_wrapper=self._client_wrapper)
        self.app_settings = AsyncAppSettingsClient(client_wrapper=self._client_wrapper)
        self.conversation = AsyncConversationClient(client_wrapper=self._client_wrapper)
        self.knowledge = AsyncKnowledgeClient(client_wrapper=self._client_wrapper)
        self.translations = AsyncTranslationsClient(client_wrapper=self._client_wrapper)
        self.triggers = AsyncTriggersClient(client_wrapper=self._client_wrapper)
        self.users = AsyncUsersClient(client_wrapper=self._client_wrapper)


def _get_base_url(*, base_url: typing.Optional[str] = None, environment: MavenAGIEnvironment) -> str:
    if base_url is not None:
        return base_url
    elif environment is not None:
        return environment.value
    else:
        raise Exception("Please pass in either base_url or environment to construct the client")



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing
from .action_parameter_type import ActionParameterType
import typing_extensions
from .action_enum_option import ActionEnumOption
from ...core.serialization import FieldMetadata
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class ActionFormField(UniversalBaseModel):
    id: str = pydantic.Field()
    """
    The ID field should be used as the key in the submitActionForm API call.
    """

    label: str = pydantic.Field()
    """
    The title of the field to show on the form.
    """

    description: str = pydantic.Field()
    """
    A longer description of the field which should be shown in smaller text near the label.
    """

    required: bool = pydantic.Field()
    """
    Whether the field is required for the action. Client side validation is recommended for an improved user experience.
    """

    suggestion: typing.Optional[typing.Optional[typing.Any]] = pydantic.Field(default=None)
    """
    A value for the field provided by the LLM. All form fields should default to this value if present.
    """

    type: ActionParameterType = pydantic.Field()
    """
    Describes how the action field should be validated. Client side validation is recommended but not required. | If `enumOptions` are provided, using a dropdown or radio field is preferred. Otherwise, if the `type` is `BOOLEAN` a checkbox is preferred. | Fallback to a text input.
    """

    enum_options: typing_extensions.Annotated[
        typing.Optional[typing.List[ActionEnumOption]], FieldMetadata(alias="enumOptions")
    ] = pydantic.Field(default=None)
    """
    The options that should be shown to the user as a dropdown or radio.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from .action_base import ActionBase
from .precondition_group import PreconditionGroup
import typing_extensions
from .entity_id import EntityId
from ...core.serialization import FieldMetadata
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing
from ...core.pydantic_utilities import update_forward_refs


class ActionResponse(ActionBase):
    """
    Examples
    --------
    from mavenagi.commons import (
        ActionResponse,
        EntityId,
        Precondition_Group,
        Precondition_User,
    )

    ActionResponse(
        action_id=EntityId(
            reference_id="get-balance",
            app_id="my-billing-system",
            organization_id="acme",
            agent_id="support",
            type="ACTION",
        ),
        name="Get the user's balance",
        description="This action calls an API to get the user's current balance.",
        user_interaction_required=False,
        user_form_parameters=[],
        precondition=Precondition_Group(
            operator="AND",
            preconditions=[
                Precondition_User(
                    key="userKey",
                ),
                Precondition_User(
                    key="userKey2",
                ),
            ],
        ),
    )
    """

    action_id: typing_extensions.Annotated[EntityId, FieldMetadata(alias="actionId")] = pydantic.Field()
    """
    ID that uniquely identifies this action
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


update_forward_refs(PreconditionGroup, ActionResponse=ActionResponse)



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..core.request_options import RequestOptions
from .types.translation_response import TranslationResponse
from ..core.pydantic_utilities import parse_obj_as
from ..commons.errors.not_found_error import NotFoundError
from ..commons.types.error_message import ErrorMessage
from ..commons.errors.bad_request_error import BadRequestError
from ..commons.errors.server_error import ServerError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class TranslationsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def translate(
        self, *, text: str, target_language: str, request_options: typing.Optional[RequestOptions] = None
    ) -> TranslationResponse:
        """
        Translate text from one language to another

        Parameters
        ----------
        text : str
            The text to translate

        target_language : str
            The target language to translate to, in ISO 639-1 code format.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TranslationResponse

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.translations.translate(
            text="Hello world",
            target_language="es",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/translations/translate",
            method="POST",
            json={
                "text": text,
                "targetLanguage": target_language,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    TranslationResponse,
                    parse_obj_as(
                        type_=TranslationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise ServerError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncTranslationsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def translate(
        self, *, text: str, target_language: str, request_options: typing.Optional[RequestOptions] = None
    ) -> TranslationResponse:
        """
        Translate text from one language to another

        Parameters
        ----------
        text : str
            The text to translate

        target_language : str
            The target language to translate to, in ISO 639-1 code format.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TranslationResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.translations.translate(
                text="Hello world",
                target_language="es",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/translations/translate",
            method="POST",
            json={
                "text": text,
                "targetLanguage": target_language,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    TranslationResponse,
                    parse_obj_as(
                        type_=TranslationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise ServerError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing_extensions
import typing
from ...core.serialization import FieldMetadata
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class Range(UniversalBaseModel):
    from_: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="from")] = pydantic.Field(
        default=None
    )
    """
    Lower bound of the range (inclusive).
    """

    to: typing.Optional[float] = pydantic.Field(default=None)
    """
    Upper bound of the range (exclusive).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
from .knowledge_base_version_type import KnowledgeBaseVersionType
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class KnowledgeBaseVersion(UniversalBaseModel):
    """
    Examples
    --------
    from mavenagi.knowledge import KnowledgeBaseVersion

    KnowledgeBaseVersion(
        type="FULL",
    )
    """

    type: KnowledgeBaseVersionType = pydantic.Field()
    """
    Indicates whether the completed version constitutes a full or partial refresh of the knowledge base. Deleting and updating documents is only supported for partial refreshes.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing_extensions
import typing
from .time_series import TimeSeries
from ...core.serialization import FieldMetadata
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class DateHistogramResponse(UniversalBaseModel):
    time_series: typing_extensions.Annotated[typing.List[TimeSeries], FieldMetadata(alias="timeSeries")] = (
        pydantic.Field()
    )
    """
    The dataset for the date histogram.
    Each series represents a separate plottable time series.
    Series names reflect the grouping field values.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing_extensions
from ...core.serialization import FieldMetadata
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class EntityIdBase(UniversalBaseModel):
    """
    All other entityId fields are inferred from the API request.
    """

    reference_id: typing_extensions.Annotated[str, FieldMetadata(alias="referenceId")] = pydantic.Field()
    """
    Externally supplied ID to uniquely identify this object. Is globally unique when combined with all other entityId fields (type, appId, organizationId, agentId)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from .types.conversation_group_by import ConversationGroupBy
from .types.conversation_column_definition import ConversationColumnDefinition
from .types.time_interval import TimeInterval
from ..conversation.types.conversation_filter import ConversationFilter
from ..core.request_options import RequestOptions
from .types.conversation_table_response import ConversationTableResponse
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.pydantic_utilities import parse_obj_as
from ..commons.errors.not_found_error import NotFoundError
from ..commons.types.error_message import ErrorMessage
from ..commons.errors.bad_request_error import BadRequestError
from ..commons.errors.server_error import ServerError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from .types.conversation_chart_request import ConversationChartRequest
from .types.chart_response import ChartResponse
from .types.feedback_group_by import FeedbackGroupBy
from .types.feedback_column_definition import FeedbackColumnDefinition
from ..conversation.types.feedback_filter import FeedbackFilter
from .types.feedback_table_response import FeedbackTableResponse
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class AnalyticsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_conversation_table(
        self,
        *,
        field_groupings: typing.Sequence[ConversationGroupBy],
        column_definitions: typing.Sequence[ConversationColumnDefinition],
        time_grouping: typing.Optional[TimeInterval] = OMIT,
        conversation_filter: typing.Optional[ConversationFilter] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConversationTableResponse:
        """
        Retrieves structured conversation data formatted as a table, allowing users to group, filter, and define specific metrics to display as columns.

        Parameters
        ----------
        field_groupings : typing.Sequence[ConversationGroupBy]
            Specifies the fields by which data should be grouped. Each unique combination forms a row.
            If multiple fields are provided, the result is grouped by their unique value combinations.
            If empty, all data is aggregated into a single row.

        column_definitions : typing.Sequence[ConversationColumnDefinition]
            Specifies the metrics to be displayed as columns. Column headers act as keys, with computed metric values as their mapped values. There needs to be at least one column definition in the table request.

        time_grouping : typing.Optional[TimeInterval]
            Defines the time interval for grouping data. If specified, data is grouped accordingly  based on the time they were created. Example: If set to "DAY," data will be aggregated by day.

        conversation_filter : typing.Optional[ConversationFilter]
            Optional filter applied to refine the conversation data before processing.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationTableResponse

        Examples
        --------
        from mavenagi import MavenAGI
        from mavenagi.analytics import (
            ConversationColumnDefinition,
            ConversationGroupBy,
            ConversationMetric_Average,
            ConversationMetric_Count,
            ConversationMetric_Percentile,
        )
        from mavenagi.conversation import ConversationFilter

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.analytics.get_conversation_table(
            conversation_filter=ConversationFilter(
                languages=["en", "es"],
            ),
            time_grouping="DAY",
            field_groupings=[
                ConversationGroupBy(
                    field="Category",
                )
            ],
            column_definitions=[
                ConversationColumnDefinition(
                    header="count",
                    metric=ConversationMetric_Count(),
                ),
                ConversationColumnDefinition(
                    header="avg_first_response_time",
                    metric=ConversationMetric_Average(
                        target_field="FirstResponseTime",
                    ),
                ),
                ConversationColumnDefinition(
                    header="percentile_handle_time",
                    metric=ConversationMetric_Percentile(
                        target_field="HandleTime",
                        percentile=25.0,
                    ),
                ),
            ],
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/tables/conversations",
            method="POST",
            json={
                "timeGrouping": time_grouping,
                "fieldGroupings": convert_and_respect_annotation_metadata(
                    object_=field_groupings, annotation=typing.Sequence[ConversationGroupBy], direction="write"
                ),
                "columnDefinitions": convert_and_respect_annotation_metadata(
                    object_=column_definitions,
                    annotation=typing.Sequence[ConversationColumnDefinition],
                    direction="write",
                ),
                "conversationFilter": convert_and_respect_annotation_metadata(
                    object_=conversation_filter, annotation=ConversationFilter, direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ConversationTableResponse,
                    parse_obj_as(
                        type_=ConversationTableResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise ServerError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_conversation_chart(
        self, *, request: ConversationChartRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> ChartResponse:
        """
        Fetches conversation data visualized in a chart format. Supported chart types include pie chart, date histogram, and stacked bar charts.

        Parameters
        ----------
        request : ConversationChartRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChartResponse

        Examples
        --------
        from mavenagi import MavenAGI
        from mavenagi.analytics import (
            ConversationChartRequest_PieChart,
            ConversationGroupBy,
            ConversationMetric_Count,
        )
        from mavenagi.conversation import ConversationFilter

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.analytics.get_conversation_chart(
            request=ConversationChartRequest_PieChart(
                conversation_filter=ConversationFilter(
                    languages=["en", "es"],
                ),
                group_by=ConversationGroupBy(
                    field="Category",
                ),
                metric=ConversationMetric_Count(),
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/charts/conversations",
            method="POST",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=ConversationChartRequest, direction="write"
            ),
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ChartResponse,
                    parse_obj_as(
                        type_=ChartResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise ServerError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_feedback_table(
        self,
        *,
        field_groupings: typing.Sequence[FeedbackGroupBy],
        column_definitions: typing.Sequence[FeedbackColumnDefinition],
        time_grouping: typing.Optional[TimeInterval] = OMIT,
        feedback_filter: typing.Optional[FeedbackFilter] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FeedbackTableResponse:
        """
        Retrieves structured feedback data formatted as a table, allowing users to group, filter,  and define specific metrics to display as columns.

        Parameters
        ----------
        field_groupings : typing.Sequence[FeedbackGroupBy]
            Specifies the fields by which data should be grouped. Each unique combination forms a row.
            If multiple fields are provided, the result is grouped by their unique value combinations.
            If empty, all data is aggregated into a single row.

        column_definitions : typing.Sequence[FeedbackColumnDefinition]
            Specifies the metrics to be displayed as columns.
            Column headers act as keys, with computed metric values as their mapped values.
            There needs to be at least one column definition in the table request.

        time_grouping : typing.Optional[TimeInterval]
            Defines the time interval for grouping data. If specified, data is grouped accordingly based on the time they were created.
             Example: If set to "DAY," data will be aggregated by day.

        feedback_filter : typing.Optional[FeedbackFilter]
            Optional filter applied to refine the feedback data before processing.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FeedbackTableResponse

        Examples
        --------
        from mavenagi import MavenAGI
        from mavenagi.analytics import (
            FeedbackColumnDefinition,
            FeedbackGroupBy,
            FeedbackMetric_Count,
        )
        from mavenagi.conversation import FeedbackFilter

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.analytics.get_feedback_table(
            feedback_filter=FeedbackFilter(
                types=["THUMBS_UP", "INSERT"],
            ),
            field_groupings=[
                FeedbackGroupBy(
                    field="CreatedBy",
                )
            ],
            column_definitions=[
                FeedbackColumnDefinition(
                    header="feedback_count",
                    metric=FeedbackMetric_Count(),
                )
            ],
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/tables/feedback",
            method="POST",
            json={
                "timeGrouping": time_grouping,
                "fieldGroupings": convert_and_respect_annotation_metadata(
                    object_=field_groupings, annotation=typing.Sequence[FeedbackGroupBy], direction="write"
                ),
                "columnDefinitions": convert_and_respect_annotation_metadata(
                    object_=column_definitions, annotation=typing.Sequence[FeedbackColumnDefinition], direction="write"
                ),
                "feedbackFilter": convert_and_respect_annotation_metadata(
                    object_=feedback_filter, annotation=FeedbackFilter, direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    FeedbackTableResponse,
                    parse_obj_as(
                        type_=FeedbackTableResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise ServerError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncAnalyticsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_conversation_table(
        self,
        *,
        field_groupings: typing.Sequence[ConversationGroupBy],
        column_definitions: typing.Sequence[ConversationColumnDefinition],
        time_grouping: typing.Optional[TimeInterval] = OMIT,
        conversation_filter: typing.Optional[ConversationFilter] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConversationTableResponse:
        """
        Retrieves structured conversation data formatted as a table, allowing users to group, filter, and define specific metrics to display as columns.

        Parameters
        ----------
        field_groupings : typing.Sequence[ConversationGroupBy]
            Specifies the fields by which data should be grouped. Each unique combination forms a row.
            If multiple fields are provided, the result is grouped by their unique value combinations.
            If empty, all data is aggregated into a single row.

        column_definitions : typing.Sequence[ConversationColumnDefinition]
            Specifies the metrics to be displayed as columns. Column headers act as keys, with computed metric values as their mapped values. There needs to be at least one column definition in the table request.

        time_grouping : typing.Optional[TimeInterval]
            Defines the time interval for grouping data. If specified, data is grouped accordingly  based on the time they were created. Example: If set to "DAY," data will be aggregated by day.

        conversation_filter : typing.Optional[ConversationFilter]
            Optional filter applied to refine the conversation data before processing.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationTableResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI
        from mavenagi.analytics import (
            ConversationColumnDefinition,
            ConversationGroupBy,
            ConversationMetric_Average,
            ConversationMetric_Count,
            ConversationMetric_Percentile,
        )
        from mavenagi.conversation import ConversationFilter

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.analytics.get_conversation_table(
                conversation_filter=ConversationFilter(
                    languages=["en", "es"],
                ),
                time_grouping="DAY",
                field_groupings=[
                    ConversationGroupBy(
                        field="Category",
                    )
                ],
                column_definitions=[
                    ConversationColumnDefinition(
                        header="count",
                        metric=ConversationMetric_Count(),
                    ),
                    ConversationColumnDefinition(
                        header="avg_first_response_time",
                        metric=ConversationMetric_Average(
                            target_field="FirstResponseTime",
                        ),
                    ),
                    ConversationColumnDefinition(
                        header="percentile_handle_time",
                        metric=ConversationMetric_Percentile(
                            target_field="HandleTime",
                            percentile=25.0,
                        ),
                    ),
                ],
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/tables/conversations",
            method="POST",
            json={
                "timeGrouping": time_grouping,
                "fieldGroupings": convert_and_respect_annotation_metadata(
                    object_=field_groupings, annotation=typing.Sequence[ConversationGroupBy], direction="write"
                ),
                "columnDefinitions": convert_and_respect_annotation_metadata(
                    object_=column_definitions,
                    annotation=typing.Sequence[ConversationColumnDefinition],
                    direction="write",
                ),
                "conversationFilter": convert_and_respect_annotation_metadata(
                    object_=conversation_filter, annotation=ConversationFilter, direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ConversationTableResponse,
                    parse_obj_as(
                        type_=ConversationTableResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise ServerError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_conversation_chart(
        self, *, request: ConversationChartRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> ChartResponse:
        """
        Fetches conversation data visualized in a chart format. Supported chart types include pie chart, date histogram, and stacked bar charts.

        Parameters
        ----------
        request : ConversationChartRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChartResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI
        from mavenagi.analytics import (
            ConversationChartRequest_PieChart,
            ConversationGroupBy,
            ConversationMetric_Count,
        )
        from mavenagi.conversation import ConversationFilter

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.analytics.get_conversation_chart(
                request=ConversationChartRequest_PieChart(
                    conversation_filter=ConversationFilter(
                        languages=["en", "es"],
                    ),
                    group_by=ConversationGroupBy(
                        field="Category",
                    ),
                    metric=ConversationMetric_Count(),
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/charts/conversations",
            method="POST",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=ConversationChartRequest, direction="write"
            ),
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ChartResponse,
                    parse_obj_as(
                        type_=ChartResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise ServerError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_feedback_table(
        self,
        *,
        field_groupings: typing.Sequence[FeedbackGroupBy],
        column_definitions: typing.Sequence[FeedbackColumnDefinition],
        time_grouping: typing.Optional[TimeInterval] = OMIT,
        feedback_filter: typing.Optional[FeedbackFilter] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FeedbackTableResponse:
        """
        Retrieves structured feedback data formatted as a table, allowing users to group, filter,  and define specific metrics to display as columns.

        Parameters
        ----------
        field_groupings : typing.Sequence[FeedbackGroupBy]
            Specifies the fields by which data should be grouped. Each unique combination forms a row.
            If multiple fields are provided, the result is grouped by their unique value combinations.
            If empty, all data is aggregated into a single row.

        column_definitions : typing.Sequence[FeedbackColumnDefinition]
            Specifies the metrics to be displayed as columns.
            Column headers act as keys, with computed metric values as their mapped values.
            There needs to be at least one column definition in the table request.

        time_grouping : typing.Optional[TimeInterval]
            Defines the time interval for grouping data. If specified, data is grouped accordingly based on the time they were created.
             Example: If set to "DAY," data will be aggregated by day.

        feedback_filter : typing.Optional[FeedbackFilter]
            Optional filter applied to refine the feedback data before processing.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FeedbackTableResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI
        from mavenagi.analytics import (
            FeedbackColumnDefinition,
            FeedbackGroupBy,
            FeedbackMetric_Count,
        )
        from mavenagi.conversation import FeedbackFilter

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.analytics.get_feedback_table(
                feedback_filter=FeedbackFilter(
                    types=["THUMBS_UP", "INSERT"],
                ),
                field_groupings=[
                    FeedbackGroupBy(
                        field="CreatedBy",
                    )
                ],
                column_definitions=[
                    FeedbackColumnDefinition(
                        header="feedback_count",
                        metric=FeedbackMetric_Count(),
                    )
                ],
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/tables/feedback",
            method="POST",
            json={
                "timeGrouping": time_grouping,
                "fieldGroupings": convert_and_respect_annotation_metadata(
                    object_=field_groupings, annotation=typing.Sequence[FeedbackGroupBy], direction="write"
                ),
                "columnDefinitions": convert_and_respect_annotation_metadata(
                    object_=column_definitions, annotation=typing.Sequence[FeedbackColumnDefinition], direction="write"
                ),
                "feedbackFilter": convert_and_respect_annotation_metadata(
                    object_=feedback_filter, annotation=FeedbackFilter, direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    FeedbackTableResponse,
                    parse_obj_as(
                        type_=FeedbackTableResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise ServerError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..commons.types.entity_id_base import EntityIdBase
from ..commons.types.app_user_identifier import AppUserIdentifier
from ..commons.types.user_data import UserData
from ..core.request_options import RequestOptions
from ..commons.types.app_user_response import AppUserResponse
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.pydantic_utilities import parse_obj_as
from ..commons.errors.not_found_error import NotFoundError
from ..commons.types.error_message import ErrorMessage
from ..commons.errors.bad_request_error import BadRequestError
from ..commons.errors.server_error import ServerError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..core.jsonable_encoder import jsonable_encoder
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class UsersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create_or_update(
        self,
        *,
        user_id: EntityIdBase,
        identifiers: typing.Sequence[AppUserIdentifier],
        data: typing.Dict[str, UserData],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AppUserResponse:
        """
        Update a user or create it if it doesn't exist.

        Parameters
        ----------
        user_id : EntityIdBase
            ID that uniquely identifies this app user

        identifiers : typing.Sequence[AppUserIdentifier]
            Used to determine whether two users from different apps are the same

        data : typing.Dict[str, UserData]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AppUserResponse

        Examples
        --------
        from mavenagi import MavenAGI
        from mavenagi.commons import AppUserIdentifier, EntityIdBase, UserData

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.users.create_or_update(
            user_id=EntityIdBase(
                reference_id="user-0",
            ),
            identifiers=[
                AppUserIdentifier(
                    value="joe@myapp.com",
                    type="EMAIL",
                )
            ],
            data={
                "name": UserData(
                    value="Joe",
                    visibility="VISIBLE",
                )
            },
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/users",
            method="PUT",
            json={
                "userId": convert_and_respect_annotation_metadata(
                    object_=user_id, annotation=EntityIdBase, direction="write"
                ),
                "identifiers": convert_and_respect_annotation_metadata(
                    object_=identifiers, annotation=typing.Sequence[AppUserIdentifier], direction="write"
                ),
                "data": convert_and_respect_annotation_metadata(
                    object_=data, annotation=typing.Dict[str, UserData], direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    AppUserResponse,
                    parse_obj_as(
                        type_=AppUserResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise ServerError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self,
        user_id: str,
        *,
        app_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AppUserResponse:
        """
        Get a user by its supplied ID

        Parameters
        ----------
        user_id : str
            The reference ID of the user to get. All other entity ID fields are inferred from the request.

        app_id : typing.Optional[str]
            The App ID of the user to get. If not provided the ID of the calling app will be used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AppUserResponse

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.users.get(
            user_id="user-0",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/users/{jsonable_encoder(user_id)}",
            method="GET",
            params={
                "appId": app_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    AppUserResponse,
                    parse_obj_as(
                        type_=AppUserResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise ServerError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self,
        user_id: str,
        *,
        app_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Deletes all identifiers and user data saved by the specified app.
        Does not modify data or identifiers saved by other apps.

        If this user is linked to a user from another app, it will not be unlinked. Unlinking of users is not yet supported.

        <Warning>This is a destructive operation and cannot be undone.</Warning>

        Parameters
        ----------
        user_id : str
            The reference ID of the user to delete. All other entity ID fields are inferred from the request.

        app_id : typing.Optional[str]
            The App ID of the user to delete. If not provided the ID of the calling app will be used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.users.delete(
            user_id="user-0",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/users/{jsonable_encoder(user_id)}",
            method="DELETE",
            params={
                "appId": app_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise ServerError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncUsersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create_or_update(
        self,
        *,
        user_id: EntityIdBase,
        identifiers: typing.Sequence[AppUserIdentifier],
        data: typing.Dict[str, UserData],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AppUserResponse:
        """
        Update a user or create it if it doesn't exist.

        Parameters
        ----------
        user_id : EntityIdBase
            ID that uniquely identifies this app user

        identifiers : typing.Sequence[AppUserIdentifier]
            Used to determine whether two users from different apps are the same

        data : typing.Dict[str, UserData]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AppUserResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI
        from mavenagi.commons import AppUserIdentifier, EntityIdBase, UserData

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.users.create_or_update(
                user_id=EntityIdBase(
                    reference_id="user-0",
                ),
                identifiers=[
                    AppUserIdentifier(
                        value="joe@myapp.com",
                        type="EMAIL",
                    )
                ],
                data={
                    "name": UserData(
                        value="Joe",
                        visibility="VISIBLE",
                    )
                },
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/users",
            method="PUT",
            json={
                "userId": convert_and_respect_annotation_metadata(
                    object_=user_id, annotation=EntityIdBase, direction="write"
                ),
                "identifiers": convert_and_respect_annotation_metadata(
                    object_=identifiers, annotation=typing.Sequence[AppUserIdentifier], direction="write"
                ),
                "data": convert_and_respect_annotation_metadata(
                    object_=data, annotation=typing.Dict[str, UserData], direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    AppUserResponse,
                    parse_obj_as(
                        type_=AppUserResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise ServerError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self,
        user_id: str,
        *,
        app_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AppUserResponse:
        """
        Get a user by its supplied ID

        Parameters
        ----------
        user_id : str
            The reference ID of the user to get. All other entity ID fields are inferred from the request.

        app_id : typing.Optional[str]
            The App ID of the user to get. If not provided the ID of the calling app will be used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AppUserResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.users.get(
                user_id="user-0",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/users/{jsonable_encoder(user_id)}",
            method="GET",
            params={
                "appId": app_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    AppUserResponse,
                    parse_obj_as(
                        type_=AppUserResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise ServerError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self,
        user_id: str,
        *,
        app_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Deletes all identifiers and user data saved by the specified app.
        Does not modify data or identifiers saved by other apps.

        If this user is linked to a user from another app, it will not be unlinked. Unlinking of users is not yet supported.

        <Warning>This is a destructive operation and cannot be undone.</Warning>

        Parameters
        ----------
        user_id : str
            The reference ID of the user to delete. All other entity ID fields are inferred from the request.

        app_id : typing.Optional[str]
            The App ID of the user to delete. If not provided the ID of the calling app will be used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.users.delete(
                user_id="user-0",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/users/{jsonable_encoder(user_id)}",
            method="DELETE",
            params={
                "appId": app_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise ServerError(
                    typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from .conversation_message_base import ConversationMessageBase
import typing_extensions
from .entity_id_base import EntityIdBase
from ...core.serialization import FieldMetadata
import pydantic
from .user_conversation_message_type import UserConversationMessageType
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class UserMessageBase(ConversationMessageBase):
    user_id: typing_extensions.Annotated[EntityIdBase, FieldMetadata(alias="userId")] = pydantic.Field()
    """
    ID that uniquely identifies the user that created this message
    """

    text: str = pydantic.Field()
    """
    The text of the message. Cannot be empty
    """

    user_message_type: typing_extensions.Annotated[UserConversationMessageType, FieldMetadata(alias="userMessageType")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...commons.types.event_trigger_base import EventTriggerBase
import typing_extensions
from ...commons.types.entity_id_base import EntityIdBase
from ...core.serialization import FieldMetadata
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class EventTriggerRequest(EventTriggerBase):
    """
    Examples
    --------
    from mavenagi.commons import EntityIdBase
    from mavenagi.triggers import EventTriggerRequest

    EventTriggerRequest(
        trigger_id=EntityIdBase(
            reference_id="store-in-snowflake",
        ),
        description="Stores conversation data in Snowflake",
        type="CONVERSATION_CREATED",
    )
    """

    trigger_id: typing_extensions.Annotated[EntityIdBase, FieldMetadata(alias="triggerId")] = pydantic.Field()
    """
    ID that uniquely identifies this event trigger
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing_extensions
from ...conversation.types.feedback_field import FeedbackField
from ...core.serialization import FieldMetadata
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class FeedbackDistinctCount(UniversalBaseModel):
    """
    Calculates the number of unique values in the specified field.
    Supports fields with list values as well.
    """

    target_field: typing_extensions.Annotated[FeedbackField, FieldMetadata(alias="targetField")] = pydantic.Field()
    """
    All the distinct values of this field will be counted.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
